import React from "react";
import { useRef, useEffect, useState } from "react";
import { useTheme } from "../hooks/useTheme";
import { tokenize } from "../utils/tokenizer";
import type { HighlighterProps, Token } from "../types";
import { ChevronDown, ChevronUp, Copy, Check } from "lucide-react";

export const SyntaxHighlighter: React.FC<HighlighterProps> = ({
  code,
  language,
  showLineNumbers = true,
  wrapLines = false,
  maxHeight,
  isDarkMode,
  expandable = false,
  className = "",
  style = {},
  customBackgroundColor,
  showCopyButton = true,
}) => {
  const { theme } = useTheme();
  const codeRef = useRef<HTMLPreElement>(null);
  const [isExpanded, setIsExpanded] = useState(!expandable);
  const [isCopied, setIsCopied] = useState(false);

  useEffect(() => {
    if (codeRef.current) {
      codeRef.current.innerHTML = "";
      const tokens = tokenize(code, language);
      if (tokens.length === 0) return;

      let lineNumber = 1;
      let lineContent = "";

      tokens.forEach((token: Token) => {
        const span = document.createElement("span");
        span.textContent = token.content;
        span.className = `token ${token.type}`;
        span.style.color = `var(--syntax-${token.type}, ${
          theme[token.type as keyof typeof theme] || theme.text
        })`;
        lineContent += token.content;

        if (token.content.includes("\n")) {
          const lines = token.content.split("\n");
          lines.forEach((line, index) => {
            if (index > 0) {
              if (showLineNumbers) {
                const lineNumberSpan = document.createElement("span");
                lineNumberSpan.textContent = `${lineNumber} `;
                lineNumberSpan.className = "line-number";
                lineNumberSpan.style.color = `var(--syntax-line-number, ${theme.lineNumber})`;
                codeRef.current?.appendChild(lineNumberSpan);
              }
              lineNumber++;
            }
            const lineSpan = document.createElement("span");
            lineSpan.textContent = line;
            lineSpan.className = `token ${token.type}`;
            lineSpan.style.color = `var(--syntax-${token.type}, ${
              theme[token.type as keyof typeof theme] || theme.text
            })`;
            codeRef.current?.appendChild(lineSpan);
            if (index < lines.length - 1) {
              codeRef.current?.appendChild(document.createElement("br"));
            }
          });
        } else {
          codeRef.current?.appendChild(span);
        }

        if (lineContent.includes("\n")) {
          lineContent = "";
        }
      });

      if (showLineNumbers && !lineContent.endsWith("\n")) {
        const lineNumberSpan = document.createElement("span");
        lineNumberSpan.textContent = `${lineNumber} `;
        lineNumberSpan.className = "line-number";
        lineNumberSpan.style.color = `var(--syntax-line-number, ${theme.lineNumber})`;
        codeRef.current?.insertBefore(
          lineNumberSpan,
          codeRef.current.firstChild
        );
      }
    }
  }, [code, language, theme, showLineNumbers]);

  const toggleExpand = () => {
    setIsExpanded(!isExpanded);
  };

  const copyToClipboard = async () => {
    if (codeRef.current) {
      const text = codeRef.current.innerText;
      await navigator.clipboard.writeText(text);
      setIsCopied(true);
      setTimeout(() => setIsCopied(false), 2000);
    }
  };

  const defaultClassName = `syntax-highlighter ${isDarkMode ? "dark" : ""}`;
  const combinedClassName = className
    ? `${defaultClassName} ${className}`
    : defaultClassName;

  return (
    <div
      className={combinedClassName}
      style={{
        position: "relative",
        fontFamily: "'Fira Code', monospace",
        fontSize: "14px",
        lineHeight: 1.5,
        borderRadius: "0.5rem",
        overflow: "hidden",
        ...style,
      }}
    >
      <style>{`
        .syntax-highlighter {
          --syntax-bg: ${
            customBackgroundColor || (isDarkMode ? theme.background : "#ffffff")
          };
          --syntax-text: ${isDarkMode ? theme.text : "#24292e"};
          --scrollbar-thumb-color: rgba(156, 163, 175, 0.5);
          --scrollbar-thumb-hover-color: rgba(156, 163, 175, 0.8);
        }
        .syntax-highlighter.dark {
          --syntax-bg: ${customBackgroundColor || theme.background};
          --syntax-text: ${theme.text};
          --scrollbar-thumb-color: rgba(75, 85, 99, 0.5);
          --scrollbar-thumb-hover-color: rgba(75, 85, 99, 0.8);
        }
        .syntax-highlighter :global(pre) {
          margin: 0;
          padding: 0;
        }
        .syntax-highlighter :global(.token) {
          transition: color 0.2s ease;
        }
        .syntax-highlighter ::-webkit-scrollbar {
          width: 8px;
          height: 8px;
        }
        .syntax-highlighter ::-webkit-scrollbar-track {
          background: transparent;
        }
        .syntax-highlighter ::-webkit-scrollbar-thumb {
          background-color: var(--scrollbar-thumb-color);
          border-radius: 4px;
        }
        .syntax-highlighter ::-webkit-scrollbar-thumb:hover {
          background-color: var(--scrollbar-thumb-hover-color);
        }
        .line-number {
          user-select: none;
          opacity: 0.5;
          margin-right: 1em;
        }
      `}</style>
      <div
        className="syntax-highlighter-content"
        style={{
          padding: "1rem",
          overflow: "auto",
          maxHeight: isExpanded ? "none" : maxHeight,
          transition: "background-color 0.2s ease, color 0.2s ease",
        }}
      >
        <pre
          ref={codeRef}
          className={`language-${language}`}
          style={{
            whiteSpace: wrapLines ? "pre-wrap" : "pre",
          }}
        />
      </div>
      {showCopyButton && (
        <button
          onClick={copyToClipboard}
          className="syntax-highlighter-copy-btn"
          style={{
            position: "absolute",
            top: "0.5rem",
            right: "0.5rem",
            padding: "0.5rem",
            borderRadius: "0.375rem",
            backgroundColor: "var(--syntax-bg)",
            color: "var(--syntax-text)",
            border: "none",
            cursor: "pointer",
            display: "flex",
            alignItems: "center",
            justifyContent: "center",
            transition: "background-color 0.2s ease",
            opacity: 0.7,
          }}
          aria-label="Copy code"
        >
          {isCopied ? <Check size={18} /> : <Copy size={18} />}
        </button>
      )}
      {expandable && (
        <button
          onClick={toggleExpand}
          className="syntax-highlighter-expand-btn"
          style={{
            width: "100%",
            display: "flex",
            alignItems: "center",
            justifyContent: "center",
            padding: "0.5rem",
            backgroundColor: "var(--syntax-bg)",
            color: "var(--syntax-text)",
            border: "none",
            cursor: "pointer",
            transition: "background-color 0.2s ease",
            opacity: 0.9,
          }}
        >
          {isExpanded ? (
            <ChevronUp className="mr-2" />
          ) : (
            <ChevronDown className="mr-2" />
          )}
          {isExpanded ? "Collapse" : "Expand"}
        </button>
      )}
    </div>
  );
};
